part of sjs;

const String T_ARRAY = 'array',
  T_BOOL = 'bool',
  T_ENUM = 'enum',
  T_INT = 'int',
  T_NULL = 'null',
  T_NUM = 'num',
  T_OBJECT = 'object',
  T_STRING = 'string';

abstract class SJSType {
  
  final String type;
  
  const SJSType(this.type);
  
  bool validate (dynamic value, [Map<String, dynamic> params]);
  
  static const SJSType ARRAY = const SJSArray (),
      BOOL = const SJSBool(),
      INT = const SJSInt(),
      ENUM = const SJSEnum(),
      NULL = const SJSNull(),
      NUM = const SJSNum(),
      OBJECT = const SJSObject(),
      STRING = const SJSString();
}

class SJSArray extends SJSType {
  
  const SJSArray() : super (T_ARRAY);
  
  bool validate (value, [Map<String, dynamic> params = null]) {
    if (value is List) {
      if (params.containsKey('min')) {
        var min = params['min'];
        if (min is int) {
          if (value.length < min) {
            return false;
          }
        } else {
          throw new ArgumentError ('min must be int');
        }
      }
      if (params.containsKey('max')) {
        var max = params['max'];
        if (max is int) {
          if (value.length > max) {
            return false;
          }
        } else {
          throw new ArgumentError ('max must be int');
        }
      }
      return true;
    } else {
      return false;
    }
  }
}

class SJSBool extends SJSType {
  
  const SJSBool() : super (T_BOOL);
  
  bool validate (value, [Map<String, dynamic> params = null]) => value is bool;
}

class SJSInt extends SJSType {
  const SJSInt() : super (T_INT);
  
  bool validate (value, [Map<String, dynamic> params = null]) {
    if (value is int) {
      if (params.containsKey('min')) {
        var min = params['min'];
        if (min is int) {
          if (value < min) {
            return false;
          }
        } else {
          throw new ArgumentError ('min must be int');
        }
      }
      if (params.containsKey('max')) {
        var max = params['max'];
        if (max is int) {
          if (value > max) {
            return false;
          }
        } else {
          throw new ArgumentError ('max must be int');
        }
      }
      return true;
    } else {
      return false;
    }
  }
}

class SJSEnum extends SJSType {
  
  const SJSEnum() : super (T_ENUM);
  
  bool validate (value, [Map<String, dynamic> params = null]) =>
      value is String ? regEnum.hasMatch(value) : false;
  
  static final RegExp regEnum = new RegExp(r'enum\((\w+(\s*,\s*)?)+\)');
}

class SJSNull extends SJSType {
  
  const SJSNull() : super (T_NULL);
  
  bool validate (value, [Map<String, dynamic> params = null]) => value == null;
}

class SJSNum extends SJSType {
  
  const SJSNum() : super (T_NUM);
  
  bool validate (value, [Map<String, dynamic> params = null]) {
    if (value is num) {
      if (params.containsKey('min')) {
        var min = params['min'];
        if (min is num) {
          if (value < min) {
            return false;
          }
        } else {
          throw new ArgumentError ('min must be num');
        }
      }
      if (params.containsKey('max')) {
        var max = params['max'];
        if (max is num) {
          if (value > max) {
            return false;
          }
        } else {
          throw new ArgumentError ('max must be num');
        }
      }
      return true;
    } else {
      return false;
    }
  }
}

class SJSObject extends SJSType {
  
  const SJSObject() : super (T_OBJECT);
  
  bool validate (value, [Map<String, dynamic> params = null]) {
    if (value is Map) {
      var type = params['type'];
      var keys = value.keys;
      
      var max = params['max'];
      if (max is int) {
        if (keys.length > max) {
          return false;
        }
      } else {
        throw new ArgumentError('max must be int');
      }
      var min = params['min'];
      if (min is int) {
        if (keys.length < min) {
          return false;
        }
      } else {
        throw new ArgumentError('min must be int');
      }
      return true;
    } else {
      return false;
    }
  }
}

class SJSString extends SJSType {
  
  const SJSString() : super (T_STRING);
  
  bool validate (value, [Map<String, dynamic> params = null]) {
    if (value is String) {
      if (params.containsKey('min')) {
        var min = params['min'];
        if (min is int) {
          if (value.length < min) {
            return false;
          }
        } else {
          throw new ArgumentError ('min must be int');
        }
      }
      if (params.containsKey('max')) {
        var max = params['max'];
        if (max is int) {
          if (value.length > max) {
            return false;
          }
        } else {
          throw new ArgumentError ('max must be int');
        }
      }
      if (params.containsKey('regex')) {
        RegExp regex = new RegExp (params['regex']);
        if (! regex.hasMatch(value)) {
          return false;
        }
      }
      return true;
    } else {
      return false;
    }
  }
}